#!/usr/bin/env ruby
#encoding=utf-8
# NOTE
# 服务器HA配置脚本
# role：可选项为master或slave。
# IP地址说明：server1和server2为实际两台服务器的IP，对应的域名为server1.com和server2.com。
# haserver为指定的虚拟IP，虚拟IP不能被其他机器占用，需要和实际服务器IP在同一网段。对应的域名为haserver.com。
#
require 'fileutils'
Options = {
  enabled:  false,
  keepalived_interface: "eth0",
  data_store: "/data/export",
  server1:  "192.168.100.238",
  server2:  "192.168.100.239",
  haserver: "192.168.100.240"
}

def put_usage
  usage = <<EOF
使用root账户运行本脚本。格式：ruby ./config_server.rb {master|slave} 。
执行本脚本之前请先设置网络、放开防火墙的限制，并将Options[:enabled]设置为true。
使用 master|slave 选项指定该服务器初始角色为主机或备机。可选项为master或slave。
请先在master上执行，再在slave上执行。
EOF
  puts usage
end

def test_before_config server
  db = system "mysql -uroot -pVM5LVDn8fe -h#{server} -e 'show master status;' > /dev/null"
  redis = system "redis-cli -h #{server} info > /dev/null"
  if !db
    puts "无法登陆#{server}的mysql服务器。请确定防火墙开启3306端口、mysql能够监听外网地址，并且其他IP使用root用户登陆。"
    puts "安装中止。"
    exit 1
  end
  if !redis
    puts "无法连接到#{server}的redis服务器。请确定防火墙开启6379端口、redis能监听外网地址。"
    puts "安装中止。"
    exit 2
  end
end

def install_ha_pkg
  puts "开始安装必要软件包。安装过程中请保持联网。"
  `yum localinstall -y rpms/wget*.rpm`
  `wget http://download.gluster.org/pub/gluster/glusterfs/LATEST/CentOS/glusterfs-epel.repo -P /etc/yum.repos.d`
  `yum localinstall -y rpms/*.rpm`
end

def set_hosts server1, server2, haserver, role
  puts "设置hosts。"
  another_server = case role
                   when "master"
                     server2
                   when "slave"
                     server1
                   end
  if system("grep server1.com /etc/hosts >> /dev/null")
    cmd1 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+server1.com\)/' + server1 + '\1/g" /etc/hosts'
    cmd2 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+server2.com\)/' + server2 + '\1/g" /etc/hosts'
    cmd3 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+haserver.com\)/' + haserver + '\1/g" /etc/hosts'
    cmd5 = 'sed -i "s/[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}.[0-9]\{1,3\}\([ \t]\+anotherserver.com\)/' + another_server + '\1/g" /etc/hosts'
    puts "更新host项。"

    `#{cmd1}`
    `#{cmd2}`
    `#{cmd3}`
    `#{cmd5}`
  else
    puts "追加host项。"
    append_hosts = <<EOF
    #{server1}          server1.com
    #{server2}          server2.com
    #{haserver}         haserver.com
    #{another_server}   anotherserver.com
EOF
    `echo '#{append_hosts}' >> /etc/hosts`
  end
end

def generate_keepalived_conf haserver, role, keepalived_interface="eth0"
  puts "生成keepalived.conf。"
  FileUtils.mkdir_p "/etc/keepalived"

  case role
  when "master"
    priority_vi_1 = "250"
  when "slave"
    priority_vi_1 = "150"
  end

  keepalived_conf = <<EOF
global_defs {
   notification_email {
     ewhine@localhost
   }
   notification_email_from Dehui Network
   smtp_server localhost
   smtp_connect_timeout 30
   router_id LVS_DEVEL
}
vrrp_instance VI_1 {
    state BACKUP
    interface #{keepalived_interface}
    virtual_router_id 51i
    priority #{priority_vi_1}
    nopreempt
    notify_master /etc/keepalived/notify_master.sh
    notify_backup /etc/keepalived/notify_backup.sh
    advert_int 1
    authentication {
      auth_type PASS
      auth_pass 1111
    }
    virtual_ipaddress {
    #{haserver}
    }
}

########Virtual Server########
virtual_server #{haserver} 8099 {
  delay_loop 6
  lb_algo rr
  lb_kind DR
  # nat_mask  255.255.255.0 # if you use linux kernel 2.2, set this line
  persistence_timeout 50
  protocol TCP
  real_server server1.com 8099 {
    weight 1
    TCP_CHECK {
      connect_timeout 3
      nb_get_retry 3
      delay_before_retry 3
    }
  }
  real_server server1.com 8099 {
    weight 3
    TCP_CHECK {
      connect_timeout 3
      nb_get_retry 3
      delay_before_retry 3
    }
  }
}
EOF

    #   keepalived_conf = keepalived_conf_orig.gsub('vip_vi_1', haserver).gsub('priority_vi_1', priority_vi_1)

    File.open('/etc/keepalived/keepalived.conf', 'w') do |f|
      f.write(keepalived_conf)
    end

    puts "生成notify_master.sh。"
    notify_master_script = <<EOF
#!/bin/bash
echo [`date`]"作为master启动" >> /home/ewhine/var/log/keepalived.log
/opt/ewhine/bin/redis-cli slaveof NO ONE
pkill -USR2 mqtt3d
EOF

    File.open('/etc/keepalived/notify_master.sh', 'w') do |f|
      f.write(notify_master_script)
    end
    `chmod +x /etc/keepalived/notify_master.sh`

    puts "生成notify_backup.sh。"
    notify_backup_script = <<EOF
#!/bin/bash
echo [`date`]"作为backup启动" >> /home/ewhine/var/log/keepalived.log
pkill -USR1 mqtt3d
/opt/ewhine/bin/redis-cli  slaveof anotherserver.com 6379
du /home/ewhine/deploy/ewhine_NB/efiles  >> /dev/null &
echo "flush_all" | /opt/ewhine/bin/nc localhost 11211 -c
EOF
    File.open('/etc/keepalived/notify_backup.sh', 'w') do |f|
      f.write(notify_backup_script)
    end
    `chmod +x /etc/keepalived/notify_backup.sh`

#     `ln -s /etc/rc3.d/S70keepalived /etc/init.d/keepalived`

    `chkconfig keepalived on`

end


def configure_mysql_replication server1, server2, role
  puts "设置server_id。"
  server_id = case role
              when "master"
                "10"
              when "slave"
                "20"
              end

  FileUtils.cp "my.cnf", "/etc/my.cnf"

  sed_cmd = 'sed -i "s/\(server-id[ \t]*=[ \t]*\)[0-9]\+/\1' + server_id + '/g" /etc/my.cnf'
  `#{sed_cmd}`
  `service mx_mysqld restart`

  puts "注册同步用户。"

  # do sth.
  `mysql -uroot -pVM5LVDn8fe -e "use mysql;update user set Host = '%' where Host = 'localhost';grant replication slave on *.* to 'slaveuser'@'%' identified by 'minxing_slaver_123';flush privileges;"`

  puts "设置mysql同步。"
  if role == "slave"
    test_before_config server1
    test_before_config server2

    `mysql -hserver1.com -uroot -pVM5LVDn8fe -e "stop slave; reset slave;"`
    `mysql -hserver2.com -uroot -pVM5LVDn8fe -e "stop slave; reset slave;"`
    `mysqldump -uroot -pVM5LVDn8fe --add-drop-table --add-locks esns_production | mysql -hanotherserver.com -uroot -pVM5LVDn8fe esns_production`
    set_mysql server1, server2
    set_mysql server2, server1
  else
    # do nothing.
  end
end

def configure_redis server1, server2, role
  if role == "slave"
    system "redis-cli  slaveof anotherserver.com 6379"
  end
end

def set_mysql origin, replica
  puts "配置从#{origin}到#{replica}的复制"
  cmd_orig = <<EOF
mysql -hreplica -uroot -pVM5LVDn8fe -e "stop slave; change master to  master_host='origin', master_user='slaveuser', master_password='minxing_slaver_123', master_log_file='logfile', master_log_pos=logpos;start slave;"
EOF

  mysql_cmd = 'mysql -h' + origin + ' -uroot -pVM5LVDn8fe -e  "show master status\G"'
  res = `#{mysql_cmd}`
  logfile = (/File: (mariadb-bin.[0-9]+)/ =~ res; $1)
  logpos = (/Position: ([0-9]+)/ =~ res; $1)

  cmd = cmd_orig.gsub("logfile", logfile).gsub("logpos", logpos.to_s).gsub("replica", replica).gsub("origin", origin)
  `#{cmd}`
  # TODO:检查slave状态是否成功启动
  system "mysql -hserver1.com -uroot -pVM5LVDn8fe -e \"grant all on esns_production.* to esns@'%' identified by 'minxing123';flush privileges;\""
  system "mysql -hserver2.com -uroot -pVM5LVDn8fe -e \"grant all on esns_production.* to esns@'%' identified by 'minxing123';flush privileges;\""
end

def configure_glusterfs_client role, server1, server2, data_store="/data/export"
  # 生成glusterfs客户端配置文件
  FileUtils.mkdir_p data_store unless File.exists? data_store
  FileUtils.chown("ewhine", "ewhine", data_store)
  FileUtils.mkdir "/home/ewhine/efiles" unless File.exists? "/home/ewhine/efiles"
  FileUtils.chown("ewhine", "ewhine", "/home/ewhine/efiles")
  system "mv  /home/ewhine/deploy/ewhine_NB/efiles/* /home/ewhine/efiles/"

#   begin
#     FileUtils.mv "/home/ewhine/deploy/ewhine_NB/efiles", "/home/ewhine/"
#   rescue Errno::EEXIST
#     #do nothing.
#   end
#   FileUtils.mkdir_p "/home/ewhine/deploy/ewhine_NB/efiles"

  puts "生成glusterfs服务端配置文件"
  glusterfsd_vol = <<EOF
volume posix
type storage/posix
option directory #{data_store}
end-volume

volume locks
type features/locks
subvolumes posix
end-volume

volume brick
type performance/io-threads
option thread-count 8
subvolumes locks
end-volume

volume server
type protocol/server
option transport-type tcp
option auth.addr.brick.allow *
subvolumes brick
end-volume
EOF
  File.open("/etc/glusterfs/glusterfsd.vol", "w") do |f|
    f.write(glusterfsd_vol)
  end

  `/etc/init.d/glusterfsd restart`

  puts "生成glusterfs客户端配置文件"
  glusterfs_vol = <<EOF
volume self
type protocol/client
option transport-type tcp
option remote-host #{server1}
option remote-subvolume brick
end-volume

volume remote
type protocol/client
option transport-type tcp
option remote-host #{server2}
option remote-subvolume brick
end-volume

volume replicate
type cluster/replicate
subvolumes self remote
end-volume

volume writebehind
type performance/write-behind
option cache-size 512KB
option flush-behind on
subvolumes replicate
end-volume

volume cache
type performance/io-cache
option cache-size 1024MB
subvolumes writebehind
end-volume
EOF

  File.open("/etc/glusterfs/glusterfs.vol", "w") do |f|
    f.write(glusterfs_vol)
  end

  `sed -i '3,$s/NORMAL/ERROR/g;3,$s/#//g'  /etc/sysconfig/glusterd`
  `sed -i '3,$s/NORMAL/ERROR/g;3,$s/#//g'  /etc/sysconfig/glusterfsd`

  if system "grep glusterfs /etc/rc.local >> /dev/null"
    # already in rc.local. do nothing.
  else
    `chkconfig mx_rainbows off`
    `chkconfig mx_sidekiq off`
    `chkconfig mx_monit off`

    start_cmd = <<EOF
glusterfs -f /etc/glusterfs/glusterfs.vol /home/ewhine/deploy/ewhine_NB/efiles/
su ewhine -c "/etc/init.d/mx_monit start"
EOF
    `echo '#{start_cmd}' >> /etc/rc.local`
  end

  `chkconfig glusterd off`
  `chkconfig glusterfsd on`
end

def replace_initd_scripts
  `cp -f initds/* /etc/init.d/`
  `cp -f monitrc /opt/ewhine/etc/monitrc`
  `cp -f minxing_ctl.rb /home/ewhine/minxing_ctl.rb`
  `service mx_monit restart`
end

def configure_memcached_ha
end

# 重启glusterfs-server keepalived，重启nginx，挂载虚拟分区
def restart_services1_and_mount
  `service glusterfsd restart` 
  `umount /home/ewhine/deploy/ewhine_NB/efiles >> /dev/null`
  `service keepalived restart`
  `glusterfs -f /etc/glusterfs/glusterfs.vol /home/ewhine/deploy/ewhine_NB/efiles/`
  `mv /home/ewhine/efiles/* /home/ewhine/deploy/ewhine_NB/efiles/`
end

# 将rails项目中的mysql/redis/sidekiq配置文件的host修改为虚拟IP并重启rails和sidekiq
def change_rails_config
  `chown ewhine:ewhine -R config/*.yml config/*.rb`
  `cp config/application.yml /home/ewhine/deploy/ewhine_NB/current/config/application.yml`
  `cp config/database.yml /home/ewhine/deploy/ewhine_NB/current/config/database.yml`
  `cp config/production.rb /home/ewhine/deploy/ewhine_NB/current/config/environments/production.rb`
  `cp config/sidekiq.rb /home/ewhine/deploy/ewhine_NB/current/config/initializers/sidekiq.rb`
  `cp config/site.conf /opt/ewhine/nginx/conf/ewhine/site.conf`
end

def update_mqtt_bin
  `cp -f mqtt3d /opt/ewhine/bin/mqtt3d`
end

case
when ["--help", "-h"].include?(ARGV.first)
  put_usage
when ARGV.first != "master" && ARGV.first != "slave"
  puts "您没有指定当前主机的初始角色。初始角色是master或slave。"
  put_usage
when Options[:enabled] != true
  puts "当前Options[:enabled]不为true，无法执行配置。请打开此脚本文件，并配置Options选项中的IP。确定选项无误后，将其中的Options[:enabled]字段设置为true，重新执行本脚本。"
else

  role = ARGV.first
  server1 = Options[:server1]
  server2 = Options[:server2]
  haserver = Options[:haserver]
  keepalived_interface = Options[:keepalived_interface]
  #   role, server1, server2, haserver = ARGV


  confirm = <<EOF
请确认以下参数：
haserver.com:\t#{haserver}
server1.com:\t#{server1}
server2.com:\t#{server2}
按y继续，其他选项退出
EOF
  puts confirm
  if "y" != STDIN.gets.chomp
    puts "已中止"
    exit
  end
  puts "开始配置"
  install_ha_pkg

  # 设置host表
  set_hosts server1, server2, haserver, role

  # 生成keepalived配置文件
  generate_keepalived_conf haserver, role, keepalived_interface

  # 修改mysql的server_id, 配置replication
  configure_mysql_replication server1, server2, role

  # 若当前为从机，则从主机同步redis的数据
  configure_redis server1, server2, role

  # 替换initd脚本
  replace_initd_scripts

  update_mqtt_bin

  # not implement.
  # configure_memcached_ha

  # 配置glusterfs客户端配置文件
  configure_glusterfs_client role, server1, server2


  # 重启glusterfs keepalived,挂载虚拟分区
  restart_services1_and_mount

  # 将rails项目中的mysql/redis/sidekiq配置文件的host修改为虚拟IP并重启rails和sidekiq
  change_rails_config

  puts "配置完毕。请您自行设置防火墙后，重新启动服务器。"
end
