#!/usr/bin/env ruby
#encoding=utf-8
require 'fileutils'

REDIS_DB_DIR   = "/home/ewhine/var/lib/redis"
BACKUP_DIR     = "/home/ewhine/minxing_backup"
EFILES_DIR     = "/home/ewhine/deploy/ewhine_NB/efiles"
              
MYSQL_USER     = "esns"
MYSQL_PWD      = "minxing123"
MYSQL_DATABASE = "esns_production"
MYSQL_BIN_DIR  = "/opt/ewhine/mysql/bin"

class Service
  attr_accessor :name, :restart_cmd, :start_cmd, :stop_cmd,  :status_cmd

  def initialize name, restart_cmd=nil, status_cmd=nil
    @name = name
    @start_cmd   =                "/etc/init.d/mx_#{name} start   > /dev/null"
    @stop_cmd    =                "/etc/init.d/mx_#{name} stop    > /dev/null"
    @restart_cmd = restart_cmd || "/etc/init.d/mx_#{name} restart > /dev/null"
    @status_cmd  = status_cmd  || "/etc/init.d/mx_#{name} status  > /dev/null"
  end

  def start
    puts "正在启动#{@name}...\n"
    if system @start_cmd
      puts "成功启动服务"
    else 
      puts "启动#{@name}失败"
    end
  end

  def stop
    puts "正在停止#{@name}...\n"
    if system @stop_cmd
      puts "成功停止服务"
    else
      puts "停止#{@name}失败"
    end
  end

  def status
    running = system @status_cmd
    if running
      puts "#{@name}正在运行"
    else
      puts "#{@name}没有运行"
    end

  end

  def restart
    if @restart_cmd != "null"
      puts "重新启动#{@name}...\n"
      system @restart_cmd
    else
      self.stop
      self.start
    end
  end
end

SERVICES = [
  Service.new("monit", nil, 'ps -ef|grep "/opt/ewhine/bin/monit"|grep -v "grep" > /dev/null'),
  Service.new("rainbows", nil, 'ps -ef|grep "rainbows master"|grep -v "grep" > /dev/null'),
  Service.new("sidekiq", "null"),
  Service.new("faye", nil, "ps -ef |grep app.js|grep -v grep > /dev/null"),
  Service.new("mxpp", nil, "ps -ef |grep  mxpp.js |grep -v grep > /dev/null"),
  Service.new("mqtt3d", "null", 'ps -ef |grep "/opt/ewhine/bin/mqtt3d" |grep -v grep > /dev/null')
]


case ARGV.first
when "upgrade"
  file_path = ARGV[1] || ""
  if ARGV[1].nil?
    puts "参数错误。用例: ./minxing_ctl.rb upgrade /path_to/ewhine_pkg.tar.gz"
    puts "若想指定nokogiri使用系统库，则增加--nokogiri_use_system_library"

  elsif File.exist? file_path
    FileUtils.rm_rf "/home/ewhine/ewhine_pkg"
    begin
      FileUtils.mv file_path, "/home/ewhine/ewhine_pkg.tar.gz"
    rescue ArgumentError
      # 文件路径相同，不做移动
    end
    nokogiri_use_system_libs = ARGV[2]

    system "tar xzf /home/ewhine/ewhine_pkg.tar.gz -C /home/ewhine"
    system "/home/ewhine/ewhine_pkg/build.rb install #{nokogiri_use_system_libs}"
  else
    puts "包文件不存在。"
  end
when "backup"
  puts "备份开始"
  print "保存redis数据"
  system "redis-cli save"
  puts "优化redis备份文件..."
  system "redis-cli bgrewriteaof"
  
  loop do
    print "."
    sleep 0.5
    break if Dir.entries(REDIS_DB_DIR).count <= 4
  end

  puts "移动备份文件至备份路径"
  current_backup_dir = if ARGV[1].nil?
                         "#{BACKUP_DIR}/#{Time.now.strftime "%Y%m%d%H%M%S"}"
                       else
                         "#{BACKUP_DIR}/#{ARGV[1].to_s}/"
                       end
  # NOTE:use File.exists? instead of Dir.exists? to avoid a bug caused by ruby version.
  FileUtils.mkdir_p current_backup_dir unless File.exists? current_backup_dir
  FileUtils.cp_r REDIS_DB_DIR, current_backup_dir
  puts "OK"


  print "备份mysql"
  3.times do |t| 
    print "."
    sleep 0.3
  end
  system "#{MYSQL_BIN_DIR}/mysqldump -u#{MYSQL_USER} -p#{MYSQL_PWD} #{MYSQL_DATABASE}  --add-locks >> #{current_backup_dir}/esns_production.sql"
  print "OK\n"

  puts "备份文件"
  FileUtils.cp_r EFILES_DIR, current_backup_dir
  puts "OK"

  puts "已备份到 #{current_backup_dir}."
when "backup_list"
  begin
    backups = Dir.entries(BACKUP_DIR).select {|n| n[0] != "."}
  rescue Errno::ENOENT
    backups = ["您还没有备份"]
  end
  output = backups.join "\n"
  puts output

when "restore"
  if ARGV.count != 2
    puts "参数错误。用例:minxing_ctl.rb restore backup_name"
  else
    target = "#{BACKUP_DIR}/#{ARGV[1]}"
   
    SERVICES.first.stop
    system "/opt/ewhine/etc/init.d/mx_redis stop"
    FileUtils.cp_r "#{target}/redis", "#{REDIS_DB_DIR}/../"
    system "/opt/ewhine/etc/init.d/mx_redis start"
    SERVICES.first.start
    
    FileUtils.rm_rf EFILES_DIR
    FileUtils.mkdir EFILES_DIR
    FileUtils.cp_r "#{target}/efiles", "#{EFILES_DIR}/../"
    
    mysql_resotre = "#{MYSQL_BIN_DIR}/mysql -u#{MYSQL_USER} -p#{MYSQL_PWD} #{MYSQL_DATABASE} < #{target}/esns_production.sql"
    system mysql_resotre
   
    clear_memcached_cmd = 'echo "flush_all"|/opt/ewhine/bin/nc -c localhost 11211'
    system clear_memcached_cmd
  end

when "status"
  SERVICES.each do |service|
    service.status
    puts "============="
  end
when "start_services"
  SERVICES.reverse_each do |service|
    service.start
    puts "============="
  end
when "stop_services"
  SERVICES.each do |service|
    service.stop
    puts "=============="
  end
when "restart_services"
  SERVICES.reverse_each do |service|
    service.restart
    puts "=============="
  end
else
  usage = <<USAGE
用例: 
  备份还原:\t./minxing_ctl.rb [backup_list| backup <backup_name> |restore <backup_name>]
  后台服务:\t./minxing_ctl.rb [status|start_services|stop_services|restart_services]
  升级server:\t./minxing_ctl.rb [upgrade </path_to/ewhine_pkg.tar.gz>]
USAGE
  puts usage
end
